﻿using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TeamchatSDK.Client;
using TeamchatSDK.Client.Attributes;
using TeamchatSDK.Client.Interfaces;

namespace SDKDemo
{
    public class NocBot
    {

        private readonly String[] NEW_TICKET_PARAMS = new String[] { "issue", "addr", "lsi", "srno" };
        private readonly String[] NEW_TICKET_LABELS = new String[] { "Issue", "Address", "LSI", "SR NO#" };
        private readonly String ASSIGNEE = "assignee";

        private readonly String[] REPORT_PARAMS = new String[] { "ticketid", "issue", "addr", "lsi", "srno", "assignee", "acceptor", "curloc", "clientloc", "status" };
        private readonly String[] REPORT_LABELS = new String[] { "Ticket", "Issue", "Address", "LSI", "SR NO", "Assigned To", "Acceptor", "Current Location", "Client Loc", "Status" };

        private readonly String[] DAILY_REPORT_PARAMS = new String[] { "ticketid", "issue", "acceptor", "status" };
        private readonly String[] DAILY_REPORT_LABELS = new String[] { "Ticket", "Issue", "Acceptor", "Status" };

        [OnKeyword("create")]
        public void onCreateKeyword(TeamchatAPI api)
        {
            Form f = api.objects().form();
            for (int i = 0; i < NEW_TICKET_PARAMS.Length; i++)
                f.addField(api.objects().input().label(NEW_TICKET_LABELS[i]).name(NEW_TICKET_PARAMS[i]));

            api.perform(
                    api.context().currentRoom().post(
                            new PrimaryChatlet().setQuestion("New call")
                            .setReplyScreen(f)
                            .setReplyLabel("Create")
                            .alias("create")));
        }

        [OnAlias("create")]
        public void onNewTicket(TeamchatAPI api)
        {

            String ticketId = createTicket(api);
            JObject ticketdata = getTicketData(api, ticketId);

            String[] parameters = api.context().currentReply().extractFields(NEW_TICKET_PARAMS);
            String reply = api.context().currentReply().raw();
            Chatlet assigmentChatlet = new PrimaryChatlet()
                                            .setQuestionHtml(getTicketHtml(ticketdata, false))
                                            .setReplyScreen(api.objects().form()
                                                                .addField(api.objects().input()
                                                                            .name(ASSIGNEE)
                                                                            .label("Assign To")
                                                                )
                                            ).setReplyLabel("Assign")
                                            .alias("assignment")
                                            .alias("ticketid:" + ticketId);
            if (ticketdata != null)
            {
                api.perform(api.context().byId(ticketdata["tlroom"].ToString()).post(assigmentChatlet));
            }
            else { api.perform(api.context().currentRoom().post(assigmentChatlet)); }
        }

        [OnAlias("assignment")]
        public void onTicketAssign(TeamchatAPI api)
        {
            String ticketId = getTicketId(api);
            JObject ticketdata = getTicketData(api, ticketId);
            ticketdata["assignee"] = api.context().currentReply().getField(ASSIGNEE);
            api.data().addField("ticketdata", ticketId, ticketdata.ToString());

            api.perform(api.context().byId(ticketdata["tlroom"].ToString())
               .post(new PrimaryChatlet()
                   .setQuestionHtml(getTicketHtml(ticketdata, false))
                   .setReplyAction("Accept", "accepted", "1")
                   .alias("accepted")
                   .alias("ticketid:" + ticketId)
                   )
               );
        }

        [OnAlias("accepted")]
        public void onAccept(TeamchatAPI api)
        {
            String ticketId = getTicketId(api);
            JObject ticketdata = getTicketData(api, ticketId);
            String sender = api.context().currentReply().senderEmail();

            ticketdata["acceptor"] = api.context().currentReply().senderName();
            ticketdata["acceptorEmail"] = sender;

            api.data().addField("ticketdata", ticketId, ticketdata.ToString());
            publishReport(api, ticketId);
            api.perform(api.context().create()
                            .add(sender).setName("NOC Bot")
                            .post(new LocationChatlet()
                                        .setQuestion("Submit your current location")
                                        .alias("currloc")
                                        .alias("ticketid:" + ticketId)));
        }

        [OnAlias("currloc")]
        public void onUpdateCurrentLocation(TeamchatAPI api)
        {
            String ticketId = getTicketId(api);
            JObject ticketdata = getTicketData(api, ticketId);
            String sender = api.context().currentReply().senderEmail();
            ticketdata["curloc"] = api.context().currentReply().getField("locname");
            api.data().addField("ticketdata", ticketId, ticketdata.ToString());

            publishReport(api, ticketId);
            api.perform(api.context().create()
                            .add(sender).setName("NOC Bot")
                            .post(new LocationChatlet()
                                        .setQuestion("Submit once reach client location")
                                        .alias("clientloc")
                                        .alias("ticketid:" + ticketId)));

        }

        [OnKeyword("setup")]
        public void onSetupKeyword(TeamchatAPI api)
        {
            api.performPostInCurrentRoom(new PrimaryChatlet()
                .setQuestion("Configure Room type")
                .setReplyScreen(api.objects().form()
                        .addField(api.objects().select().name("reptype").label("Room Type").addOption("noc").addOption("tl"))
                        .addField(api.objects().input().name("repcode").label("Email of TL"))
                        )
                .alias("setup")
            );
        }

        [OnAlias("setup")]
        public void onSetup(TeamchatAPI api)
        {
            String roomid = api.context().currentRoom().getId();
            String type = api.context().currentReply().getField("reptype");
            String code = api.context().currentReply().getField("repcode");

            if (type.Equals("noc"))
            {
                api.data().addField("noc1", roomid, code);
            }
            else if (type.Equals("tl"))
            {
                try
                {
                    String prevcode = api.data().getField("noc1", roomid);
                    api.data().addField("noc2", prevcode, null);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
                api.data().addField("noc1", roomid, code);
                api.data().addField("noc2", code, roomid);
            }
        }

        [OnAlias("clientloc")]
        public void onUpdateClientLocation(TeamchatAPI api)
        {
            String ticketId = getTicketId(api);
            JObject ticketdata = getTicketData(api, ticketId);
            String sender = api.context().currentReply().senderEmail();
            ticketdata["clientloc"] = api.context().currentReply().getField("locname");
            api.data().addField("ticketdata", ticketId, ticketdata.ToString());
            publishReport(api, ticketId);

            api.perform(api.context().create()
                    .add(sender).setName("NOC Bot")
                    .post(new PrimaryChatlet()
                                .setQuestion("Update status for Ticket:" + ticketId)
                                .setReplyScreen(api.objects().form()
                                        .addField(api.objects().select()
                                                    .name("status")
                                                    .label("Status")
                                                    .addOption("Issue Resolved")
                                                    .addOption("Issue Not Resolved")
                                                    .addOption("Issue Pending Resolved")
                                        ).addField(api.objects().input().name("remarks").label("Remarks"))
                                )
                                .alias("feedback")
                                .alias("ticketid:" + ticketId)));
        }
        [OnAlias("feedback")]
        public void onFeedback(TeamchatAPI api)
        {
            String ticketId = getTicketId(api);
            JObject ticketdata = getTicketData(api, ticketId);
            ticketdata["status"] = api.context().currentReply().getField("status") + " " + api.context().currentReply().getField("remarks");
            api.data().addField("ticketdata", ticketId, ticketdata.ToString());
            publishReport(api, ticketId);
        }

        private String getTicketId(TeamchatAPI api)
        {
            String[] aliases = api.context().currentChatletAlias();
            if (aliases == null)
                return null;
            foreach (String a in aliases)
            {
                if (a.StartsWith("ticketid:"))
                    return a.Substring(9);
            }
            return null;
        }

        private JObject getTicketData(TeamchatAPI api, String ticketId)
        {
            String data = api.data().getField("ticketdata", ticketId);
            if (data != null)
                return new JObject(data);
            return null;
        }

        private String getTicketHtml(JObject data, bool includeEmpty)
        {
            String html = "<div>";
            try
            {
                for (int i = 0; i < REPORT_PARAMS.Length; i++)
                {
                    if (data[REPORT_PARAMS[i]] != null)
                        html += "<div>" + REPORT_LABELS[i] + ": " + data[REPORT_PARAMS[i]].ToString() + "</div>";
                    else if (includeEmpty)
                        html += "<div>" + REPORT_LABELS[i] + ": </div>";
                }
            }
            catch (Exception e) { }
            html += "</div>";
            return html;
        }

        private void publishReport(TeamchatAPI api, String ticketId)
        {
            JObject ticketdata = getTicketData(api, ticketId);
            String reportformid = null;

            if (ticketdata["reportformid"] != null)
                reportformid = ticketdata["reportformid"].ToString();
            else
            {
                reportformid = Guid.NewGuid().ToString().Replace("-", "").ToLower();
                ticketdata["reportformid"] = reportformid;
            }

            api.data().addField("ticketdata", ticketId, ticketdata.ToString());

            PrimaryChatlet p = new PrimaryChatlet().setQuestionHtml(getTicketHtml(ticketdata, true));
            BypassChatletCopy report = new BypassChatletCopy(p);
            report.setFormId(reportformid);

            JArray publishtoRooms = (JArray)ticketdata["publishtorooms"];

            for (int i = 0; i < publishtoRooms.Count; i++)
            {
                api.perform(api.context().byId(publishtoRooms[i].ToString()).post(report));
            }
        }

        private String getTLRoom(TeamchatAPI api)
        {
            //return api.data().getField("NOCtoTL", api.context().currentRoom().getId());
            try
            {
                return api.data().getField("noc2", api.data().getField("noc1", api.context().currentRoom().getId()));
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                return null;
            }
        }
        private String createTicket(TeamchatAPI api)
        {
            api.data().addToField("ticksequence", "ticketid", 1);
            String ticketId = api.data().getField("ticksequence", "ticketid");
            String nocroom = api.context().currentRoom().getId();
            String tlroom = getTLRoom(api);

            if (tlroom == null)
            {
                //TODO
                return null;
            }

            JArray publishtoRooms = new JArray();
            publishtoRooms.Add(nocroom);
            publishtoRooms.Add(tlroom);

            JObject ticketdata = new JObject();
            ticketdata.Add("publishtorooms", publishtoRooms);
            ticketdata.Add("ticketid", ticketId);
            ticketdata.Add("nocroom", nocroom);
            ticketdata.Add("tlroom", tlroom);

            for (int i = 0; i < NEW_TICKET_PARAMS.Length; i++)
                ticketdata[NEW_TICKET_PARAMS[i]] = api.context().currentReply().getField(NEW_TICKET_PARAMS[i]);
            addToTicketList(api, ticketId);
            api.data().addField("ticketdata", ticketId, ticketdata.ToString());
            return ticketId;
        }


        private void addToTicketList(TeamchatAPI api, String ticketId)
        {
            //Format formatter = new SimpleDateFormat("yyyy-MM-dd");		
            String date = DateTime.Now.ToString("yyyy-MM-dd");
            JArray tlist = null;
            String list = api.data().getField("ticketlist", date);

            if (list == null)
            {
                tlist = new JArray();
            }
            else
            {
                tlist = new JArray(list);
            }

            tlist.Add(ticketId);
            api.data().addField("ticketlist", date, tlist.ToString());
        }

        public static void Main(String[] args)
        {
            TeamchatAPI api = TeamchatAPIImpl.fromFile("teamchat.data")
                    .setEmail("bot@teamchat.com")
                    .setPassword("botpwd");

            //api.data().addField("NOCtoTL", "551b8e2793cd89f65bbf9612", "551b8f7f93cd89f65bbf9617");	

            api.context().byId("5588561d915456295e15d12c").post(new TextChatlet("Hello.."));
            api.startReceivingEvents(new NocBot());

            //Console.ReadKey();
        }

        [OnKeyword("report")]
        public void onReport(TeamchatAPI api)
        {
            String date = DateTime.Now.ToString("yyyy-MM-dd");
            JArray tlist = null;
            String list = api.data().getField("ticketlist", date);

            if (list == null)
                return;

            tlist = new JArray(list);
            publishDailyReport(api, tlist);
        }

        private void publishDailyReport(TeamchatAPI api, JArray tickets)
        {
            String roomid = api.context().currentRoom().getId();
            int ticketcount = 0;


            String html = "<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\">";

            html += "<tr>";
            for (int j = 0; j < DAILY_REPORT_LABELS.Length; j++)
                html += "<td>" + DAILY_REPORT_LABELS[j] + "</td>";
            html += "</tr>";

            for (int i = 0; i < tickets.Count; i++)
            {
                JObject data = getTicketData(api, tickets[i].ToString());

                if (!(data["nocroom"].ToString().Equals(roomid) || data["tlroom"].ToString().Equals(roomid)))
                    continue;

                ticketcount++;
                html += "<tr>";
                for (int j = 0; j < DAILY_REPORT_PARAMS.Length; j++)
                {
                    if (data[DAILY_REPORT_PARAMS[j]] != null)
                        html += "<td>" + data[DAILY_REPORT_PARAMS[j]].ToString() + "</td>";
                    else
                        html += "<td> </td>";
                }
                html += "</tr>";
            }
            html += "</table>";

            if (ticketcount == 0)
                html = "No tickets for today";

            PrimaryChatlet p = new PrimaryChatlet().setQuestionHtml(html);
            api.perform(api.context().currentRoom().post(p));
        }
    }
}
