﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TeamchatSDK.Client;
using TeamchatSDK.Client.Attributes;
using TeamchatSDK.Client.Interfaces;

namespace SDKDemo
{
    public class PostReplyDemo
    {
        public static readonly String bot = "bot@teamchat.com";
        public static readonly String password = "botpwd";

        public static void Main(String[] args)
        {
            TeamchatAPI api = TeamchatAPIImpl.fromFile("config.json")
                    .setEmail(bot)
                    .setPassword(password);

            api.startReceivingEvents(new PostReplyDemo());
            Console.ReadKey();
        }

        /*
         * method will be called when user sends "postreply" keyword
         */
        [OnKeyword("postreply")]
        public void handleReply(TeamchatAPI api)
        {
            Chatlet[] chatlet = api.context().chatletByAlias("profileform");

            if (chatlet == null || !(chatlet.Length > 0))
            {
                api.performPostInCurrentRoom(new TextChatlet("Create the profile form first"));
                return;
            }

            api.perform(
                    api.context().currentRoom().reply(chatlet[0].createReply()
                            .addField("name", "Random Name")
                            .addField("city", "Mumbai")
                )
            );
        }

        /*
         * This method will be called when user sends a keyword "create" in the room
         */
        [OnKeyword("create")]
        public void onCreate(TeamchatAPI api)
        {
            PrimaryChatlet p = new PrimaryChatlet().setQuestion("Please enter your profile")
                    .setReplyScreen(api.objects().form()
                        .addField(api.objects().input().name("name").label("Name"))
                        .addField(api.objects().select().name("city").label("City")
                                .addOption("Delhi").addOption("Mumbai")
                                .addOption("Kolkata").addOption("Chennai")))
                    .showDetails(true)
                    .setDetailsLabel("Entries")
                    .allowComments(true)
                    .setReplyLabel("Enter");

            p.alias("profileform");
            api.performPostInCurrentRoom(p);
        }
    }
}
